package org.datamanager.engine.cluster;

import java.util.*;
import java.io.*;

import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;

import org.datamanager.engine.filter.*;


/**
 * This class represents the only attributes of the document that engine cares about.
 *
 * @author Aleksander Slominski
 * @author Chao
 */


public class EngineDocument implements EntityValue {

    /** @serial filename*/ 
    private String name;

    /** @serial url */
    private String url;

    /** @serial content of the document*/
    private String content;

    /** @serial WeightedWordList from document*/
    private WeightedWordList wwl;

    /** @serial */
    private Coordinates _coord;

    /** @serial */
    private Entity associatedEntity;
    
    /** constructor takes a string name, a string url, a string content and an entity*/
    EngineDocument(String name, String url, String content, Entity entity) {
	this(name, url, content);
	this.associatedEntity = entity;
    }

    /**
     * Creates new engine document with given name, url and content 
     * 
     * Constructor takes only a string name, string url and a string content
     */
    EngineDocument(String name, String url, String content) {
       this.name = name;
       this.url = url;
       this.content = content;
       updateFiltering();
       updateCoord();

    }

    /** method returns the Coordinates */
    public synchronized Coordinates getCoordinates() {
	return _coord;
    }

    /** method returns the distance between two documents */
    public synchronized double getDistanceTo(EngineDocument ed) {
	double distance = 0;
	if(_coord != null) {
	    if(_coord.size() != ed.getCoordinates().size()) {
		updateCoord();
		ed.updateCoord();
	    }
	    distance = _coord.getDistanceTo(ed.getCoordinates());
 	}
	return distance;
    }

    public Entity getEntity() {
	return associatedEntity;
    }
    
    public synchronized double getSimilarityTo(EngineDocument ed) {
	double similarity = 0;
	if(_coord != null) {
	    if(_coord.size() != ed.getCoordinates().size()) {
		updateCoord();
		ed.updateCoord();
	    }
	    similarity = _coord.getSimilarityTo(ed.getCoordinates());
 	}
	return similarity;
    }

    /** return the name of the document */
    public String getName() {
	return name;
    }

    public String getUrl() {
	return url;
    }

    public String getContent() {
	return content;
    }


    /** method returns the weightedWordList of the document */
    public WeightedWordList getWeightedWordList() {
	return wwl;
    }


    public synchronized void updateFiltering() {
       wwl = (WeightedWordList) EngineManager.getDefault().getFilter().process(content);
    }

    /**
     * Update coordinates of this document to accomodate new sematic space.
     */
    public synchronized void updateCoord() {
       EngineManager em = EngineManager.getDefault();
       _coord = em.computeCoord(wwl);
    }

    public String toString() {
	return name;//+ " = " + _coord;
    }
}
