package org.datamanager.engine.cluster;

import java.util.*;
import java.io.*;
import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.exception.DataManagerException;
import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;
import org.datamanager.engine.*;
import org.datamanager.engine.filter.*;
import org.datamanager.engine.tools.*;

/**
 * Class EngineManager is the main class that basically manages everything.
 *
 * @author Aleksander Slominski
 */

public class EngineManager implements EntityValue {
    /** @serial */
    private List _clusters = new Vector(100, 100);
     /** @serial */
    private List _docs = new Vector(100, 100);
    /** @serial */
    private Map _dosc2clusters = new Hashtable();
    /** @serial */
    private Filter _filter;
    /** @serial */
    private double _similarityThreshold = LSA.SIMILARITY_THRESHOLD;

    /** @serial */
    private LSA _lsa = new LSA();
    /** @serial */
    private List _lsaTerms;
    /** @serial */
    private Map _lsaTerm2index;



    private static EngineManager _instance = new EngineManager();

    /**
     * Construct empty EngineManager, can't be instantiated directly.
     */
    protected EngineManager(){
	//set default filter
	CompositeFilter cf = new CompositeFilter();
	cf.addFilter(new HTMLFilter());
	cf.addFilter(new DetWordFilter());
	cf.addFilter(new StopWordFilter());
	cf.addFilter(new FrequencyFilter());
	_filter = cf;
    }


    /**
     * Get one and only default engine manager.
     */
    public static EngineManager getDefault() {
	return _instance;
    }

    /*
    public static void setDefault(EngineManager e) {
	_instance = e;
    }
    */

    //Factories go here


    /**
     * Creates cluster that can be anaged by engine.
     */
    public EngineCluster createEngineCluster() {
	return new EngineCluster();
    }

    /**
     * Create engine document associted to given Entity.
     */
    public EngineDocument createEngineDocument(String name, String url, String content, Entity entity) {
       EngineDocument ed = new EngineDocument(name, url, content, entity);
       addEngineDocument(ed);
       return ed;
    }

    /**
     * Create engine document with given name, url and content.
     */
    public EngineDocument createEngineDocument(String name, String url, String content) {
       EngineDocument ed =  new EngineDocument(name, url, content);
       addEngineDocument(ed);
       return ed;
    }


    /**
     * Construct document from file fileName.
     * @throws IOException
     */
    public EngineDocument createEngineDocument(String fileName) throws IOException {
       StringBuffer buf = new StringBuffer();
       BufferedReader bufferReader = new BufferedReader(new FileReader(fileName));
       String line = null;
       while ( (line = bufferReader.readLine() ) != null ) {
	   buf.append(line+"\n");
       }
       String name = fileName.substring(fileName.lastIndexOf('/')+1);
       String url = "file:"+fileName;
       String content = buf.toString();
       EngineDocument ed = new EngineDocument(name, url, content);
       addEngineDocument(ed);
       return ed;
    }



    public EngineDocument getDocumentFromEntity(Entity e) {
	EngineDocument ed = null;
	try {
	    Entity[] edAttr = e.getAttributes(new NameEqualsConstraint(Engine.ENGINE_DOCUMENT));
	    if(edAttr == null || edAttr.length == 0)
		return null;
	    ed = (EngineDocument) edAttr[0].getValue();
	    Assertion.check(ed != null);
	} catch (DataManagerException ex) {
	    throw new EngineException("removeEntities exception:"+ex);
	}
	return ed;
    }



    public void addEngineDocument(EngineDocument ed) {
	if(!_docs.contains(ed)) {
	    _docs.add(ed);
	}
    }

    public void removeEngineDocument(EngineDocument ed) {
	_docs.remove(ed);
    }


    /**
     * Add an Enginecluster to list of clusters managed by EngineManager.
     */
    public void addCluster(Cluster c) {
	EngineCluster ec = (EngineCluster) c;
	_clusters.add(ec);
	for(Iterator i = ec.iterator(); i.hasNext(); ) {
	    EngineDocument ed = (EngineDocument) i.next();
	    List ld = (List) _dosc2clusters.get(ed);
	    if(ld == null) {
		ld = new Vector();
		_dosc2clusters.put(ed, ld);
		//updateSpace();
	    }
	    if(!ld.contains(ec)) {
		ld.add(ec);
	    }
	    // implicitly add cluster documents to list of maintained docs
	    addEngineDocument(ed);
	}
    }

    public void removeCluster(EngineCluster c) {
	EngineCluster ec = (EngineCluster) c;
	_clusters.remove(ec);
	for(Iterator i = ec.iterator(); i.hasNext(); ) {
	    EngineDocument ed = (EngineDocument) i.next();
	    List ld = (List) _dosc2clusters.get(ed);
	    if(ld.contains(ec)) {
		ld.remove(ec);
	    }
	    if(ld.isEmpty()) {
		_dosc2clusters.remove(ed);
		//updateSpace();
	    }
	}
    }

    public Iterator getClusters() {
	return _clusters.iterator();
    }


    public EngineCluster getCluster(int i) {
	return (EngineCluster) _clusters.get(i);
    }

    public int getClustersSize() {
	return _clusters.size();
    }

    //public Iterator getDocFromCluster(EngineCluster c) {
    //	int i = _clusters.indexOf(c);
    //	return (i == -1) ? new Vector().iterator() : ((List)_clusters.get(i)) .iterator();
    //}


    public int getDocumentsSize() {
	return _docs.size();
    }

    public Iterator getDocuments() {
	return _docs.iterator();
    }

    public boolean isDocClustered(EngineDocument ed) {
	return _dosc2clusters.get(ed) != null;
    }


    public double getSimilarityThreshold() {
	return _similarityThreshold;
    }

    public void  setSimilarityThreshold(double t) {
	Assertion.check(t > 0);
	Assertion.check(t <= 1);
	_similarityThreshold = t;
    }

    public Filter getFilter() {
	return _filter;
    }


    public void setFilter(Filter f) {
	_filter = f;
	//refilter all docs and update their coords
	for(int i = 0; i < _docs.size(); ++i) {
	    EngineDocument ed = (EngineDocument) _docs.get(i);
	    ed.updateFiltering();
	    ed.updateCoord();
	}
    }

    public EngineCluster[] getClustersFromDoc(EngineDocument ed) {
	EngineCluster [] ecl = new EngineCluster[0];
	if(_dosc2clusters.get(ed) != null) {
	    List list = (List) _dosc2clusters.get(ed);
	    //ecl = new EngineCluster[list.size()];
	    if(list != null) {
		ecl = (EngineCluster[]) list.toArray(ecl);
		Assertion.check(ecl.length == list.size());
	    }
	}
	return ecl;
    }

    /**
     * Read files from directory dir with extenstion ext and returns list of EngineDocument.
     * @throws IOException
     */
    public EngineDocument[] readDocuments(String dir, final String ext) throws IOException {
	File f = new File(dir);
	String[] s = f.list(new FilenameFilter() {
	    public boolean accept(File dir,String name) {
	    	return name.endsWith(ext);
	    } });
	Vector list = new Vector();
	for(int i = 0; s!= null && i < s.length;++i) {
	    EngineDocument ed = createEngineDocument(dir+s[i]);
	    list.addElement(ed);
	}
	EngineDocument docs[] = new EngineDocument[list.size()];

	for(int i =  0; i < docs.length; ++i) {
	    docs[i] = (EngineDocument) list.elementAt(i);
	}


	return docs;
    }

    /**
     * Perform a search using specific keywords then returns an EngineCluster.
     */
    public EngineCluster search(String keywords[]) {
	EngineCluster results = new EngineCluster();

	/*
	for(int cj = 0; cj < _clusters.size(); ++cj) {
	    EngineCluster c = (EngineCluster) _clusters.get(0);
	    for(int i = 0; i < c.size(); ++i) {
		EngineDocument ed = c.getDocument(i);
		WeightedWordList wwl = ed.getWeightedWordList();
		for(int k = 0; k < keywords.length; ++k) {
		    if(wwl.get(keywords[k]) != null) {
			results.addDocument(ed); //TODO
			break;
		    }
		}
	    }
	}
	*/

	for(int i = 0; i < _docs.size(); ++i) {
	    EngineDocument ed = (EngineDocument) _docs.get(i);
	    WeightedWordList wwl = ed.getWeightedWordList();
	    for(int k = 0; k < keywords.length; ++k) {
		if(wwl.get(keywords[k]) != null) {
		    results.addDocument(ed); //TODO
		    break;
		}
	    }
	}
	return results;
    }

    public synchronized void updateSpace() {
	//List _lsaDocs = new Vector(_dosc2clusters.keySet());
	int countDoc = _docs.size();
	trace("update space called countDoc="+countDoc);
	if(countDoc == 0)
	    return;
	// We want to cut dimensions significatnly
	_lsaTerm2index = new TreeMap();
	_lsaTerms = new Vector();
	// check all docs and fill terms list
	int countTerm = 0;
	for(int i = 0; i < countDoc; i++) {
	   EngineDocument ed = (EngineDocument) _docs.get(i);
	   WeightedWordList wwl = ed.getWeightedWordList();
	   for(Iterator it = wwl.values().iterator(); it.hasNext(); ) {
	       WeightedWord ww = (WeightedWord) it.next();
	       String s = ww.getWord();
	       if(_lsaTerm2index.get(s) == null) {
		   _lsaTerm2index.put(s, new Integer(countTerm++));
		   _lsaTerms.add(s);
	       }
	   }
	}
	Assertion.check(_lsaTerms.size() == countTerm);
	// check if LSA can work - enough distinct terms
	trace("updateSpace has countTerm="+countTerm+" countDoc="+countDoc);
	if(countTerm >= countDoc) {
	    // create coincidence matrix
	    double[][] vals = new double [countTerm][];
	    for(int i = 0; i < countTerm; ++i) {
		String term = (String) _lsaTerms.get(i);
		Assertion.check( ((Integer)_lsaTerm2index.get(term)).equals(new Integer(i)) );
		vals[i] = new double[countDoc];
		for(int j = 0; j < countDoc; ++j) {
		    EngineDocument ed = (EngineDocument) _docs.get(j);
		    WeightedWord ww = (WeightedWord) ed.getWeightedWordList().get(term);
		    if(ww != null) {
			Assertion.check(ww.getWeight() != 0);
			vals[i][j] = ww.getWeight();
		    } else {
			vals[i][j] = 0;
		    }
		}
	    }
	    // update LSA mapping
	    _lsa.setDimensions( countDoc / 30 );
	    _lsa.recomputeSVD(vals, countTerm, countDoc);
	    // recompute coordinates
	    for(int i = 0; i < countDoc; ++i) {
		EngineDocument ed = (EngineDocument) _docs.get(i);
		ed.updateCoord();
	    }
	}

    }



    public synchronized Coordinates computeCoord(WeightedWordList wwl) {
	if( _lsaTerms == null || _lsaTerms.size() == 0)
	    return null;
	double[] terms = new double[ _lsaTerms.size() ];
	Assertion.check(terms[0] == 0); // array must be zero initialize
	for(Iterator i = wwl.values().iterator(); i.hasNext(); ) {
	    WeightedWord ww = (WeightedWord) i.next();
	    String s = ww.getWord();
	    Integer ii = (Integer)_lsaTerm2index.get(s);
	    if(ii != null) {
		int index = ii.intValue();
		terms[index] = ww.getWeight();
	    }
	}
	double[] result = _lsa.query(terms);
	Coordinates coord = new Coordinates(result);
	return coord;
    }

    private void writeObject(ObjectOutputStream stream) throws IOException {
	stream.defaultWriteObject();
	stream.writeObject(_instance);
	trace("engine manager serialized");
    }

    private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
	stream.defaultReadObject();
	_instance = (EngineManager) stream.readObject();
	//_instance = this;
	trace("engine manager deserialized");
    }

    private void trace(String s) {
	Trace.println(this.getClass(), s);
    }

}
