package org.datamanager.engine.simpleton;

import java.util.*;

import org.datamanager.tools.Information;
import org.datamanager.passiveentityvalue.*;
import org.datamanager.constraint.*; import org.datamanager.exception.*;
import org.datamanager.event.*;
import org.datamanager.kernel.*;

import org.datamanager.engine.*;
import org.datamanager.engine.cluster.*;
import org.datamanager.engine.tools.*;

/**
 * This simpleton mission is to cluster all new documents.
 * It clusters documets that were send to it by DocumentDownloadedEvent.
 *
 * When clustering fires events:<ul>
 * <li>new clusters are created.
 * <li>EngineClusteredEntityEvent : entites are added to existing clusters
 * </ol>
 *
 * @author Aleksander Slominski
 *
 * @see org.datamanager.engine.cluster.EngineManager
 * @see org.datamanager.event.EngineClusterCreatedEvent
 * @see org.datamanager.event.EngineClusteredEntityEvent
 */
public class ClusteringSimpleton extends Simpleton implements EventHandler {

    public ClusteringSimpleton() {
    }

    public Information getInformation() {
	String[] authors = {"Aleksander Slominski"};
	return new Information("2-ClusteringSimpleton","short", "long", "1.0", authors);
    }

    /**
     * Register this simpleton to listen for DocumentDownloadedEvent.
     * Then periodically do clustering - instead of doing it for each document.
     * That would be real overkill to call LSA for each new document.
     */
    public void process() {
	trace("clustering started");
	org.datamanager.tools.Debug.setDebugger(false);
	try {
	    // make sure engine will be serialized

	    Pool pool = Pool.getDefaultPool();
	    Assertion.check(pool != null);
	    Entity[] el = null;
	    try {
		el = pool.search(new NameContainsConstraint(Engine.ENGINE_NAME));
	    } catch(NullPointerException ex) {
		trace("NPE can't find engine?");
	    }
	    if(el == null || el.length == 0) {
		Entity manager = Entity.create(Engine.ENGINE_NAME);
		Assertion.check(Engine.getDefault() != null);
		manager.setValue(Engine.getDefault());
		pool.add(manager);
		trace("engine created");
	    } else {
		//serialization works?
		EntityValue val = el[0].getValue();
		if(val instanceof Engine) {
		  Engine engine = (Engine) val;
		  //Engine.setDefault(engine);
		  Assertion.check(engine == Engine.getDefault());
		  trace("engine was restored correctly");
		} else {
		  trace("engine was NOT restored correctly");
		}
	    }


	    // wait for documents
	    EventConstraint constraint = new EventTypeIsConstraint(DocumentDownloadedEvent.class);
	    Pool.getDefaultPool().subscribe(this, constraint);


	    Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
	    // periodical clustering and sematic Space updating
	    EngineManager em = EngineManager.getDefault();
	    int oldCountDoc = 0;
	    while(true) {
		try {
		    Thread.sleep(10000);
		} catch(InterruptedException ex){
		}
		if(oldCountDoc != countDoc) {
		    trace("updating semantic space countDoc="+countDoc+" oldCountDoc="+oldCountDoc);
		    oldCountDoc = countDoc;
		    em.updateSpace();
		    trace("updating done");
		}
		synchronized(Engine.class) {
		    trace("start reclustering");
		    for(Iterator i = em.getDocuments(); i.hasNext(); ) {
			EngineDocument ed = (EngineDocument) i.next();
			//TODO: better disriminator or characteristic doc for cluster
			if(!em.isDocClustered(ed)) {
			    try {
				clusterize(ed);
			    } catch(DataManagerException ex) {
				trace("process clustering:"+ex);
			    }
			}
		    }
		    trace("done reclustering");
		}
	    }

	}
	catch (DataManagerException ex) {
	    trace("process exception:"+ex);
	}
    }

    private static  int countDoc = 0;
    private static  int countCluster = 0;

    /**
     * Prepare just downloaded document from event to be clustered by Engine.
     */
    public synchronized void handle(DataManagerEvent event) {
	synchronized(Engine.class) {
	    try {
		Pool pool = Pool.getDefaultPool();
		DocumentDownloadedEvent downloadEvent = (DocumentDownloadedEvent) event;
		Entity doc = downloadEvent.getEntity();
		pool.add(doc);

		// Create info that Engine needs
		String name = doc.getName();
		String url = "engine://"+name + "/" + ++countDoc;
		Entity[] urlAttr = doc.getAttributes(new NameEqualsConstraint("content-location"));
		if(urlAttr.length > 0) {
		    url = urlAttr[0].getValue().toString();
		}
		String content = doc.getValue().toString();
		//trace("clustering name="+name+" url="+url+" doc="+doc);
		EngineManager em = EngineManager.getDefault();
		EngineDocument ed = em.createEngineDocument(name, url, content, doc);
		Entity engineDoc = Entity.create(Engine.ENGINE_DOCUMENT);
		engineDoc.setValue(ed);
		pool.add(engineDoc);
		doc.attach(engineDoc);

		//trace("entity clustering started for name="+name);

		/*
		if(count < 10 || (count < 100 && count % 10 == 9) || (count > 100 && count % 50 == 50)) {
		    em.updateSpace();
		}
		if(em.getDocumentsSize() > 1) {
		    clusterize(ed);
		}
		*/
	    }
	    catch (DataManagerException ex) {
		trace("handle exception:"+ex);
	    }
	}
    }

    /**
     * Do clustering for all documents that were submitted to Engine but
     * not yet clustered.
     */
    private void clusterize(EngineDocument ed) throws DataManagerException {
		    // clusterize documents
	Entity doc = ed.getEntity();
	Pool pool = Pool.getDefaultPool();
	EngineManager em = EngineManager.getDefault();
	EngineCluster potentialCluster = em.createEngineCluster();
	potentialCluster.addDocument(ed);
	//trace("ed url="+ed.getUrl()+" content="+ed.getContent());
	for(Iterator i = em.getDocuments(); i.hasNext(); ) {
	    EngineDocument other = (EngineDocument) i.next();
	    //trace("similarity "+ed.getUrl()+" to "+other.getUrl()+" is="+ed.getSimilarityTo(other));
	    if(ed.getSimilarityTo(other) > em.getSimilarityThreshold()) {
		if(em.isDocClustered(other)) {
		    potentialCluster = null; //no need to create new cluster
		    EngineCluster ecl[] = em.getClustersFromDoc(other);
		    Assertion.check(ecl.length >= 0);
		    for(int k = 0; k < ecl.length; ++k) {
			if(! ecl[k].containsDocument(ed)) {
			    ecl[k].addDocument(ed);
			    Entity cluster = ecl[k].getEntity();
			    cluster.attach(doc);
			    EngineClusteredEntityEvent ev = new EngineClusteredEntityEvent(pool, doc, cluster);
			    //trace("firing "+ev);
			    pool.add(ev);
			}

		    }
		} else if(potentialCluster != null) {
		    // we do not want greedy clusters :-)
		    //potentialCluster.addDocument(other);
		}
	    }
	}
	if(potentialCluster != null) { // && potentialCluster.size() >= 1) {
	    potentialCluster.setName("C" + ++countCluster);
	    Entity cluster = Entity.create("engine-cluster-"+potentialCluster.getName());
	    potentialCluster.setEntity(cluster);
	    em.addCluster(potentialCluster);
	    cluster.setValue(potentialCluster);
	    pool.add(cluster);
	    for(Iterator i = potentialCluster.iterator(); i.hasNext();) {
		EngineDocument ced = (EngineDocument) i.next();
		cluster.attach(ced.getEntity());
	    }
	    EngineClusterCreatedEvent createdEvent = new EngineClusterCreatedEvent(pool, cluster);
	    //trace("firing "+createdEvent);
	    pool.add(createdEvent);
	}
	//trace("entity clustering done");
    }

    private void trace(String s) {
	Trace.println(this.getClass(), s);
    }
}
