package org.datamanager.kernel;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.kernel.Entity;
import org.datamanager.kernel.DataManagerThread;
import org.datamanager.event.DataManagerEvent;
import org.datamanager.tools.Debug;

/**
 * This class is an abstract EntityConstraint class. It implements the
 * EntityConstraint interface located in org.datamanager.constraint,
 * and provides a public abstract describes() methods, which is exactly
 * the same as the public abstract accepts() method in EntityConstraint.
 * Every EntityConstraint should ideally extend this class instead of
 * implementing EntityConstraint themselves. By doing so the constraint
 * will gain extra power while working with Entities in the kernel.
 * For example, a when a constraint is working on an Entity, calling
 * methods such as getValue() and getAttributes(), it should not
 * cause DataManagerEvents to be fired. If the constriant is a
 * subclass of AbstractEntityConstraint then no events will be fired,
 * otherwise events will.
 *
 * If a constraint has already been written to implement EntityConstraint
 * it can easily be modified to use this class instead simply by changing
 * implements EntityConstraint to extends AbstractEntityConstraint and
 * renaming the accepts() method to describes().
 *
 * @author Matthew Farrellee
 */
public abstract class AbstractEntityConstraint
	implements EntityConstraint
   {

   /**
    * The accepts() method required by the EntityConstraint interface.
    * This class implements this method and makes it final, so
    * no subclasses can override the work that is done by this
    * class. Eventually this accepts() method will call the
    * abstract describes() method that should be implemented by
    * subclasses.
    */
   public final boolean accepts(Entity entity)
      {
      // Make the DataManagerThread not event-generating.
      // If the thread is not an DataManagerThread
      // there should be a DataManagerException.
      try
         {
         DataManagerThread thread =
               (DataManagerThread) Thread.currentThread();

         boolean oldValue = thread.isEventGenerating();

         thread.generateEvents(false);

         boolean validity = describes(entity);

         // Make the DataManagerThread back to the way it
         // was.
         thread.generateEvents(oldValue);

         return validity;
         }
      catch (ClassCastException exception)
         {
         // This should really never happen. If it has
         // then it means that someone is running a thread
         // of their own that is not valid.
         // The thread should really be killed at this point,
         // but maybe it would be better just to report the
         // rogue thread to the ThreadManager.

         Debug.assert("illegal thread running in the system: " +
               Thread.currentThread());
         }

      // This constraint will just return false if the thread
      // is invalid.
      return false;
      }

   /**
     * The method required by this class. This method has the
     * exact same semantics as the interface EntityConstraint's
     * accepts() method.
     */
   public abstract boolean describes(Entity entity);


   }
