/**
 * EntityProxy.java
 *
 * Created FROG_KNOWS
 */

package org.datamanager.kernel;

import org.datamanager.kernel.EntityValue;
import org.datamanager.kernel.EventGeneratorAssistant;
import org.datamanager.kernel.Entity;
import org.datamanager.kernel.Identifier;

import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;

import org.datamanager.exception.DataManagerException;

import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.GetValueEvent;
import org.datamanager.event.SetValueEvent;
import org.datamanager.event.AttachLinkEvent;
import org.datamanager.event.DetachLinkEvent;
import org.datamanager.event.SubscribeEvent;
import org.datamanager.event.UnsubscribeEvent;

import org.datamanager.passiveentityvalue.NullEntityValue;

import org.datamanager.tools.Debug;


/**
 * EntityProxies are what the Kernel deals out to Simpletons requesting
 * Entities.  They have an Identifier which they feed to the EntityManager
 * to delegate their methods to an appropriate Entity implementation for
 * the current object store.
 *
 * @author Matthew Farrellee
 * @author Benjamin Keil
 */
class EntityProxy extends Entity
	{
	private Identifier identifier;

	private EntityManager manager = EntityManager.getInstance();

	/**
	 * Delegate all subscription issues to this assistant.
	 */
	private transient EventGeneratorAssistant assistant;


	/**
	 * Create an Entity with the given name.
	 */
	EntityProxy(Identifier identifier)
		{
		this.identifier = identifier;
		}


	protected Identifier getIdentifier()
		{
		return identifier;
		}


	protected VerySimpleEntity getProxiedEntity()
		{
		return EntityManager.getInstance().findEntity(identifier);
		}

	protected void loadNewAssistant()
		{
		assistant = new EventGeneratorAssistant();
		}

	protected void fireEvent(DataManagerEvent event)
		{
		if (assistant == null)
			{
			loadNewAssistant();
			}

		assistant.fireEvent(event);
		}

	///////////////////////////////////////////////////////////////////////
	//
	//						 Entity Linkage
	//
	///////////////////////////////////////////////////////////////////////

	/**
	 * Attach the given Entity as an attribute.  Relies on a private method
	 * call to make a reciprocating link back from the attribute.
	 */
	public void attach(Entity attribute)
		throws DataManagerException
		{
		Debug.assert(attribute != null, "Attaching a null attribute!");

		getProxiedEntity().attach(((EntityProxy) attribute).getIdentifier());
		fireEvent(new AttachLinkEvent((EventGenerator) this, attribute));
		attribute.addBase(this);
		}

	/**
	  * Add an Entity to the list of bases.  This should only be called when
	  * the base is adding this Entity as an attribute, that is, you should
	  * probably *NEVER* make an explicit call to this function.
	  */
	protected void addBase(Entity base)
		{
		Debug.assert(base != null, "Adding a null base!");

		getProxiedEntity().addBase(((EntityProxy) base).getIdentifier());
		fireEvent(new AttachLinkEvent((EventGenerator) this, base));
		}

	/**
	  * Detach the given Entity.
	  */
	public void detach(Entity attribute)
		throws DataManagerException
		{
		Debug.assert(attribute != null, "Detaching a null attribute!");

		getProxiedEntity().detach(((EntityProxy) attribute).getIdentifier());
		fireEvent(new DetachLinkEvent((EventGenerator) this, attribute));
		attribute.removeBase(this);
		}

	/**
	  * Remove an Entity from the list of bases.  This should only be called
	  * by the base as it removes this Entity as an attribute (which it does
	  * automatically), that is, you should probably *NEVER* write a call to
	  * this method explicitly.
	  */
	protected void removeBase(Entity base)
		{
		Debug.assert(base != null, "Removing a null base!");

		getProxiedEntity().removeBase(((EntityProxy) base).getIdentifier());
		fireEvent(new DetachLinkEvent((EventGenerator) this, base));
		}

	///////////////////////////////////////////////////////////////////////
	//
	//							Entity Access
	//
	///////////////////////////////////////////////////////////////////////

	/**
	 * Request the Entity's name.
	 */
	public String getName()
		throws DataManagerException
		{
		return getProxiedEntity().getName();
		}

	/**
	  * Request the Entity's value.
	  */
	public EntityValue getValue()
		throws DataManagerException
		{
		fireEvent(new GetValueEvent((EventGenerator) this));
		EntityValue result = getProxiedEntity().getValue();
		if (result instanceof ActiveEntityValue)
			{
			return ((ActiveEntityValue) result).getEntityValue();
			}
		else
			{
			return result;
			}
		}

	/**
	  * Request a change of the Entity's value.
	  */
	public void setValue(EntityValue value)
		throws DataManagerException
		{
		EntityValue oldValue = getProxiedEntity().getValue();
		EntityValue newValue;

		if (value == null)
			{
			newValue = new NullEntityValue();
			}
		else
			{
			newValue = value;
			}

		getProxiedEntity().setValue(newValue);
		fireEvent(new SetValueEvent((EventGenerator) this, oldValue, newValue));
		}

	/**
	 * Request a constrained set of all entities linked to the Entity.
	 */
	public Entity[] getAttributes(EntityConstraint constraint)
		throws DataManagerException
		{
		return getProxiedEntity().getAttributes(constraint);
		}

	/**
	  * Request a constrained set of all entities of which this Entity
	  * is an attribute.
	  */
	public Entity[] getBases(EntityConstraint constraint)
		throws DataManagerException
		{
		return getProxiedEntity().getBases(constraint);
		}

	///////////////////////////////////////////////////////////////////////
	//
	//					  Entity Subscription
	//
	///////////////////////////////////////////////////////////////////////

	/**
	 * Subscribe to the Entity for DataManagerEvents satisfying the
	 * event constraint.
	 */
	public void subscribe(EventHandler listener,
			EventConstraint heard)
		throws DataManagerException
		{
		if (assistant == null)
			{
			loadNewAssistant();
			}

		assistant.addEventHandler(listener, heard);
		fireEvent(new SubscribeEvent((EventGenerator) this, listener, heard));
		}

	/**
	  * Remove all event subscriptions the Simpleton has requested
	  * from this Entity.
	  */
	public void unsubscribe(EventHandler handler)
		throws DataManagerException
		{
		if (assistant == null)
			{
			loadNewAssistant();
			}

		fireEvent(new UnsubscribeEvent((EventGenerator) this, handler));
		assistant.removeEventHandler(handler);
		}

	public String toString()
		{
		return getProxiedEntity().toString();
		}


	public int hashCode()
		{
		return identifier.hashCode();
		}

	public boolean equals(Object object)
		{
		try
			{
			return (identifier.equals(((EntityProxy) object).getIdentifier()));
			}
		catch (ClassCastException exception)
			{
			return false;
			}
		}

	} // End EntityProxy.java