/**
 * VerySimpleEntity.java
 *
 * Created 199911191500
 */

package org.datamanager.kernel;

import java.util.Vector;
import java.io.Serializable;
import org.datamanager.passiveentityvalue.NullEntityValue;
import org.datamanager.kernel.EntityValue;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.exception.DataManagerException;

import org.datamanager.tools.Debug;

/**
 * VerySimpleEntities are always in memory -- they are never cached on
 * disk.
 *
 * @see VerySimpleEntityManager#  
 *
 * @author Benjamin Keil
 * @author Matthew Farrellee
 */

class VerySimpleEntity
	implements Serializable
	{
	private String name;
	private EntityValue value;
	private Vector attributes;
	private Vector bases;

	///////////////////////////////////////////////////////////////////////
	//
	//						 Entity Creation
	//
	///////////////////////////////////////////////////////////////////////

	/**
	  * Create an Entity with the given name.
	  */
	protected VerySimpleEntity(String name)
		{
		this.name = name;
		value = new NullEntityValue();
		attributes = new Vector(5, 5);
		bases = new Vector(5, 5);
		}


	///////////////////////////////////////////////////////////////////////
	//
	//						  Entity Linkage
	//
	///////////////////////////////////////////////////////////////////////

	/**
	 * Attach the given Entity as an attribute.  Relies on a private method
	 * call to make a reciprocating link back from the attribute.
	 */
	public void attach(Identifier attribute)
		throws DataManagerException
		{
		Debug.assert(attribute != null, "Given a null Identifier!");

		synchronized (attributes)
			{
			if (bases.contains(attribute))
				throw new DataManagerException(attribute + " " 
				+ "already attached as base!");
			if (!attributes.contains(attribute))
				{
				attributes.add(attribute);
				}
			}
		}

	/**
	  * Add an Entity to the list of bases.  This should only be called when
	  * the base is adding this Entity as an attribute, that is, you should
	  * probably *NEVER* make an explicit call to this function.
	  */
	protected void addBase(Identifier base)
		{
		Debug.assert(base != null, "Given a null Identifier!");

		synchronized (bases)
			{
			bases.add(base);
			}
		}

	/**
	  * Detach the given Entity.
	  */
	public void detach(Identifier attribute)
		throws DataManagerException
		{
		Debug.assert(attribute != null, "Removing a null attribute!");

		synchronized (attributes)
			{
			if (attributes.contains(attribute))
				{
				attributes.remove(attribute);
				}
			}
		}

	/**
	  * Remove an Entity from the list of bases.  This should only be called
	  * by the base as it removes this Entity as an attribute (which it does
	  * automatically), that is, you should probably *NEVER* write a call to
	  * this method explicitly.
	  */
	protected void removeBase(Identifier base)
		{
		Debug.assert(base != null, "Removing a null base!");

		synchronized (bases)
			{
			bases.remove(base);
			}
		}

	///////////////////////////////////////////////////////////////////////
	//
	//							Entity Access
	//
	///////////////////////////////////////////////////////////////////////

	/**
	 * Request the Entity's name.
	 */
	public String getName()
		throws DataManagerException
		{
		return name;
		}

	/**
	  * Request the Entity's value.
	  */
	public EntityValue getValue()
		throws DataManagerException
		{
		return value;
		}

	/**
	  * Request a change of the Entity's value.
	  */
	public void setValue(EntityValue newValue)
		throws DataManagerException
		{
		Debug.assert(newValue != null, "Setting EntityValue to null!");

		synchronized (value)
			{
			this.value = newValue;
			}
		}

	/**
	  * Request a constrained set of all entities linked to the Entity.
	  */
	public Entity[] getAttributes(EntityConstraint constraint)
		throws DataManagerException
		{
		Debug.assert(constraint != null, "Trying to use a null constraint.");

		synchronized (attributes)
			{
			return chooseEntities(constraint, attributes);
			}
		}

	/**
	  * Request a constrained set of all entities of which this Entity
	  * is an attribute.
	  */
	public Entity[] getBases(EntityConstraint constraint)
		throws DataManagerException
		{
		Debug.assert(constraint != null, "Trying to use a null constraint.");

		synchronized (bases)
			{
			return chooseEntities(constraint, bases);
			}
		}


	protected Entity[] chooseEntities(EntityConstraint constraint,
												 Vector store)
		{
		Debug.assert(constraint != null, "Trying to use a null constraint.");
		Debug.assert(store != null, "Trying to search a null group of entities.");

		Vector accepted = new Vector();
		for (int i = 0 ; i < store.size(); i++)
			{
			Entity defendant =
				EntityManager.getInstance().findProxy((Identifier) store.elementAt(i));

			if (constraint.accepts(defendant))
				{
				accepted.add(defendant);
				}
			}

		return getEntities(accepted);
		}

	protected Entity[] getEntities(Vector vector)
		{
		Entity[] returnables = new Entity[vector.size()];
		for (int i = 0 ; i < vector.size(); i++)
			{
			Debug.assert(vector.elementAt(i) != null,
				"Found a null that passed a constraint.");
			returnables[i] = (Entity) vector.elementAt(i);
			}

		Debug.assert(returnables != null, "Results of search on '" +
			this + "' is null!");
		
		return returnables;
		}


	public String toString()
		{
		return "Entity[" +
				 "[name: " + name + "]" +
				 "[value: " + value + "]" + "]";
		}
	}
