package org.datamanager.userinterface.cerulean.controller.tree;

import org.datamanager.userinterface.cerulean.model.tree.EntityTreeModel;
import org.datamanager.userinterface.cerulean.model.event.ClusterEvent;
import org.datamanager.userinterface.cerulean.model.event.EntityEvent;
import org.datamanager.userinterface.cerulean.model.event.ClusterContentEvent;
import org.datamanager.userinterface.cerulean.model.event.ClusterReclusteringEvent;
import org.datamanager.userinterface.cerulean.model.event.EntityAndClusterAdapter;
import org.datamanager.userinterface.cerulean.model.InterfaceCluster;
import org.datamanager.userinterface.cerulean.model.data.storage.ClusterList;

/**
 *This class is resposible for listening for Cluster and Entity events for
 * the class EntityTreeModel.  It listens for six of the possible Cluster and 
 *Entity events: clusterSelected, clusterUnselected, entitiesAddedToCluster,
 *entitiesRemovedFromCluster,entitiesReclustered, and entityModified.  When
 *the events are triggered, it calls the right method in the EntityTreeModel
 *that it was created in.  NOTE: Several of these method just remove the node
 *and then readd it get the changes.  This should be rewritten so that it only
 *removes what it needs to.  To do this, however, will require some rewritting 
 *of EntityTreeModel as well. 
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indiana.edu)
 *
 *@see org.datamanager.userinterface.cerulean.model.tree.EntityTreeModel
 *@see org.datamanager.userinterface.cerulean.model.event.ClusterEvent
 *@see org.datamanager.userinterface.cerulean.model.event.ClusterContentEvent
 *@see org.datamanager.userinterface.cerulean.model.event.ClusterReclusteringEvent
 *@see org.datamanager.userinterface.cerulean.model.event.EntityEvent
 *@see org.datamanager.userinterface.cerulean.model.event.ClusterAdapter
 *@see org.datamanager.userinterface.cerulean.model.InterfaceCluster
 *@see org.datamanager.userinterface.cerulean.model.InterfaceClusterManager
 */
//  Methods:
//  ~~~~~~~~
//  public ClusterSelectionListener(EntityTreeModel)
//          sets the model field
//  public void clusterSelected(ClusterEvent)
//          calls the addCluster(InterfaceCluster) method of the model with the
//          InterfaceCluster from the ClusterEvent.
//  public void clusterUnselected(ClusterEvent)
//          calls the removeCluster(InterfaceCluster) methode of the model with
//          the InterfaceCluster from the ClusterEvent.  We know that if its 
//          unselected that it must have first been selected, so no test is
//          done first (NOTE: if you change this you must also change this
//          method).
//  public void entitiesAddedToCluster(ClusterContentEvent)
//          Adds the entities (retrived though the event) to the cluster
//  public void entitiesRemovedFromCluster(ClusterContentEvent)
//          calls the removeCluster(InterfaceCluster) then the
//          addCluster(InterfaceCluster) methods of the model with the
//          InterfaceCluster from the ClusterContentEvent.  NOTE: This
//          method should at some point it time be made more efficent
//  public void entitiesReclustered(ClusterReclusteringEvent)
//         This method removes the entites from the old clusters, and 
//         adds them to the new one.  NOTE: This should be changed so that
//         it does not remove the old cluster completly. 
//  public void entityModified(EntityEvent)
//         This removes the cluster and readds it when an entity is changed.
//         NOTE: Should be changed so it does not remove the entire cluster.
/////////////////////////////////////////////////////////////////////


public class ClusterSelectionListener extends EntityAndClusterAdapter
   {
   //this model this listener controls
   private EntityTreeModel model;

   ////////////////////////////////////////////////////////
   /**
    * This is the only constructor
    *@param model Sets the model field
    */
   public ClusterSelectionListener(EntityTreeModel model)
      {
      this.model = model;
      }

   ////////////////////////////////////////////////////////
   /**
    *This is called by InterfaceClusterManager when a cluster is selected.
    *@param event Fired when the cluster is selected.
    */
   public void clusterSelected(ClusterEvent event)
      {
      model.addCluster(event.getCluster());
      }

   ////////////////////////////////////////////////////////
   /**
    *This is called by the InterfaceClusterManager when a cluster is
    *unselected.  We don't have to worry about checking the model contians 
    *this cluster, since to be unselected it has to first be unselected. 
    *@param event Fired when the cluster is unselected.
    */
   public void clusterUnselected(ClusterEvent event)
      {
      model.removeCluster(event.getCluster());
      }

   ////////////////////////////////////////////////////////
   /**
    *This is called by the InterfaceClusterManager when an group (or a single)
    *entities have been added to a cluster
    *@param event Fired when a group of entities have been added to the cluster
    */
   public void entitiesAddedToCluster(ClusterContentEvent event) 
      {
	  if(model.contains(event.getCluster()))
	      {
		  //We do this instead of adding and removing to prevent the
		  //tree from looking horrible when a large group is added
		  model.addEntitiesToCluster(event.getCluster(), 
					     event.getEntities());
	      }
      }

   ////////////////////////////////////////////////////////////////////
   /**
    *This is called when a group (or single)of entities are removed from 
    *the cluster. NOTE: This should be rewritten sometime in the future. It 
    *would be better if only the entities being removed were removed instead 
    *of the whole cluster.
    *@param event Fired when a group of entities are removed from a cluster.
    */
   public void entitiesRemovedFromCluster(ClusterContentEvent event) 
      {
	  if(model.contains(event.getCluster()))
	      {
		  model.removeCluster(event.getCluster());
		  model.addCluster(event.getCluster());
	      }
      }

   ///////////////////////////////////////////////////////////////////
   /**
    *This method is called when a user reclusters a group of entities.
    *NOTE: This method should be rewritten so that it does not remove the 
    *entire cluster and readd it. 
    *@param event Fired when a user reclusters a group of entities.
    */
   public void entitiesReclustered(ClusterReclusteringEvent event) 
      {
	ClusterList clusters = event.getOldClusters();
	InterfaceCluster newCluster = event.getNewCluster();
	for (int i = 0; i < clusters.size(); i++) 
	    {
		InterfaceCluster cluster = clusters.elementAt(i);
		if(model.contains(cluster))
		    {
			model.removeCluster(cluster);
			model.addCluster(cluster);
		    }
	    }
	if(model.contains(newCluster))
	    {
		model.addEntitiesToCluster(newCluster, event.getEntities());
	    }
      }

   /////////////////////////////////////////////////////////////////
   /**
    *This method is called when the user changes the state of the entity 
    *in someway.
    *@param event Fired when the user changes an InterfaceEntity.
    */
   public void entityModified(EntityEvent event)
      {
	  InterfaceCluster cluster = event.getEntity().getParentCluster();
	  if(model.contains(cluster))
	      {
		  model.removeCluster(cluster);
		  model.addCluster(cluster);
	      }
      }
  }//end of class
