/*
class org.datamanager.userinterface.cerulean.model.InterfaceCluster
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/

package org.datamanager.userinterface.cerulean.model;

import org.datamanager.userinterface.cerulean.controller.entity.InterfaceClusterEntityListener;
import org.datamanager.userinterface.cerulean.model.InterfaceCluster;
import org.datamanager.userinterface.cerulean.model.InterfaceEntity;
import org.datamanager.userinterface.cerulean.model.InterfaceEntityManager;
import org.datamanager.userinterface.cerulean.model.event.InterfaceClusterEventManager;
import org.datamanager.userinterface.cerulean.model.event.ClusterListener;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.data.storage.ClusterList;
import org.datamanager.passiveentityvalue.VectorEntityValue;
import org.datamanager.kernel.EntityValue;
import org.datamanager.kernel.Entity;
import org.datamanager.tools.Debug;
import java.util.Vector;
import java.util.Hashtable;
import java.io.Serializable;
/**
 * InterfaceCluster is a collection of InterfaceEntityies grouped together.  It maintains its
 * own events which other objects may register to listen for.  Entities may
 * be added to and removed from the cluster at any time.
 *
 * @see InterfaceEntity
 * @see InterfaceEntityList
 * @see InterfaceClusterEventManager
 * @see InterfaceEntityManager
 * @see InterfaceClusterListener
 *
 * @version 1.0
 * @author Gordon Murphy, gpmurphy@indiana.edu
 * @author Bryan Dawson, bdawson@indiana.edu 
 */

public class InterfaceCluster implements Serializable {
	private Vector cluster;
	private String name;
	private InterfaceClusterEventManager eventManager;
	private InterfaceEntityList entityList;
	private boolean isSelected, isDissolvable;
	private InterfaceEntity poolCluster;
////////////////////////////////////////////////////////////////////////////////
/**
 * Creates a new InterfaceCluster with an no name, and an empty InterfaceEntityList
 */
	public InterfaceCluster() {
		this("");
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Creates a new InterfaceCluster with the specified name, and an empty InterfaceEntityList
 * @param name the name of the cluster
 */
	public InterfaceCluster(String name) {
		this(name, new InterfaceEntityList());
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Creates a new InterfaceCluster with the specified name and containing the specified
 * EntityList
 * @param name the name of the cluster
 * @param list the EntityList that the cluster represents
 */
	public InterfaceCluster(String name, InterfaceEntityList entityList) {
		setEntityList(entityList);
		//set the name of the cluster
		setName(name);
		//NOTE: commented out for now, this will be added again later.
		//createPoolCluster();
		//create
		eventManager = new InterfaceClusterEventManager();

		//register the cluster with the cluster manager
		InterfaceClusterManager.getInstance().registerCluster(this);

		//add an entity listener to the cluster manager
		InterfaceEntityManager entityManager = InterfaceEntityManager.getInstance();
		entityManager.addEntityListener(new InterfaceClusterEntityListener(this));
		setDissolvable(true);
		}
/*
	public InterfaceCluster(InterfaceEntity entity) {
		this("");
		setEntityList(getEntitiesFromPoolCluster(entity));
		}
*/
	public void createPoolCluster() {
		poolCluster = new InterfaceEntity(getName());
		InterfaceAttribute attribute = new InterfaceAttribute("InterfaceCluster");
		poolCluster.attachAttribute(attribute);
		}
	public InterfaceEntityList getEntitiesFromPoolCluster(InterfaceEntity entity) {		
		InterfaceEntityList list = new InterfaceEntityList();
		if (entity.isAttributeAttached("InterfaceCluster")) {
			Debug.assert("its a pool cluster ***********************");
			poolCluster = entity;
			InterfaceAttribute attribute = entity.getAttribute("InterfaceCluster");
			EntityValue value = attribute.getValue();
			VectorEntityValue vectorValue;
			if (value instanceof VectorEntityValue) {
				vectorValue = (VectorEntityValue)value;
                        Vector vector = vectorValue.getVector();
				for (int i = 0; i < vector.size(); i++) {
					InterfaceEntity tempEntity = new InterfaceEntity((Entity)(vector.elementAt(i)));
					list.addEntity(entity);
					}
				}
			setName(entity.getName());
			}
		return list;
		}
/**
 * Adds a InterfaceEntity to the cluster.  Fires an EntitiesAddedEvent.
 * @param entity the InterfaceEntity to add
 */
	public void addEntity(InterfaceEntity entity) {
		InterfaceEntityList list = new InterfaceEntityList();
		list.addEntity(entity);
		this.addEntities(list, true);
		}
/**
 * Adds InterfaceEntities to the cluster.
 * @param entities the InterfaceEntity to add
 * @param isEventFired whether or not an event should be fired
 */
	public void addEntities(InterfaceEntityList entities, boolean isEventFired) {
		this.addEntities(this, entities, isEventFired);
		}	
/**
 * Adds InterfaceEntities to the cluster.  Fires an EntitiesAddedToClusterEvent.
 * @param entity the InterfaceEntity to add
 */
	public void addEntities(InterfaceEntityList entities) {
		this.addEntities(entities, true);
		}
/**
 * Adds InterfaceEntities to the cluster.  Fires an EntitiesAddedToClusterEvent.
 * @param entity the InterfaceEntity to add
 */
	public void addEntities(Object source, InterfaceEntityList entities, boolean isEventFired) {
		InterfaceEntity entity;
		for (int i = 0; i <= entities.size() - 1; i++) {
			entity = entities.elementAt(i);
			entity.setParentCluster(this);
			entityList.addEntity(entity);
			Debug.assert("adding "+entity);
			}
		if (isEventFired)
			eventManager.fireEntitiesAddedToCluster(source, this, entities);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Tests if the specified InterfaceEntity is contained within the cluster.
 * @param entity an entity
 */
	public boolean contains(InterfaceEntity entity) {
		return entityList.contains(entity);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes the specified InterfaceEntity from the cluster.
 * @param entity the entity to remove
 */
	public void removeEntity(InterfaceEntity entity) {
		InterfaceEntityList list = new InterfaceEntityList();
		list.addEntity(entity);
		this.removeEntities(list, true);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes the specified InterfaceEntity from the cluster.
 * @param entity the entity to remove
 * @param isEventFired whether or not the cluster should fire an event
 */
	public void removeEntities(InterfaceEntityList entities, boolean isEventFired) {
		this.removeEntities(this, entities, isEventFired);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes the specified InterfaceEntity from the cluster.
 * @param the source of the request
 * @param entity the entity to remove
 * @param isEventFired whether or not the cluster should fire an event
 */
	public void removeEntities(Object source, InterfaceEntityList entities, boolean isEventFired) {
		InterfaceEntity entity;
		for (int i = 0; i <= entities.size() - 1; i++) {
			entity = entities.elementAt(i);
			entityList.removeEntity(entity);
			Debug.assert("removing "+entity);
			}
		if (isEventFired)
			eventManager.fireEntitiesRemovedFromCluster(source, this, entities);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds the entityList to the InterfaceCluster.  Also sets all of the contained
 * entities parent cluster to the current cluster.
 */
	private void setEntityList(InterfaceEntityList entityList) {
		for (int i = 0; i <= entityList.size() - 1; i++) {
			entityList.elementAt(i).setParentCluster(this);
			}
		this.entityList = entityList;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Returns a InterfaceEntityList that contains all of the InterfaceEntityies in the cluster.
 * @return a list of InterfaceEntityies
 */
	public InterfaceEntityList getEntities() {
		return entityList;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns a InterfaceEntityList that contains all of the InterfaceEntityies in the cluster.
 * @return a list of InterfaceEntityies
 */
	public InterfaceEntityList getInterfaceEntityList() {
		return entityList;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the name of the cluster.
 * @return the name of the cluster
 */
	public String getName() {
		return name;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the name of the cluster.
 * @param name the name of the cluster
 */
	public void setName(String name) {
		this.name = name;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns whether the cluster may be dissolved or not.
 */
	public boolean isDissolvable() {
		return isDissolvable;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Sets whether the cluster may be dissolved.  Dissolving a cluster would be
 * like releasing its contents to be reclustered.
 * @param isDissolvable true if the cluster may be dissolved, otherwise false
 */
	public void setDissolvable(boolean isDissolvable) {
		this.isDissolvable = isDissolvable;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Tests whether the Cluster is selected or not
 * @return true if the cluster is selected, false otherwise
 */
	public boolean isSelected() {
		return isSelected;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Register the specified ClusterListener to receive ClusterEvents
 * @param listener the ClusterListener to register
 */
	public void addClusterListener(ClusterListener listener) {
		eventManager.addClusterListener(listener);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Unregister the specified ClusterListener 
 * @param listener the ClusterListener to unregister
 */
	public void removeClusterListener(ClusterListener listener) {
		eventManager.removeClusterListener(listener);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Move an entity from its old cluster to the current one.
 * @param entity the entity to move.
 */
	public void reclusterEntity(InterfaceEntity entity) {
		InterfaceEntityList entities = new InterfaceEntityList();
		entities.addEntity(entity);
		this.reclusterEntities(this, entities);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Move some entities from their old cluster to the current one.
 * @param entities the entities to move.
 */
	public void reclusterEntities(InterfaceEntityList entities) {
		this.reclusterEntities(this, entities, true);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Move some entities from their old cluster to the current one.
 * @source the source of the request
 * @param entities the entities to move.
 */
	public void reclusterEntities(Object source, InterfaceEntityList entities) {
		this.reclusterEntities(source, entities, true);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Move some entities from their old cluster to the current one.
 * @param source the source of the request
 * @param entities the entities to move.
 * @param isEventFired whether of not this method should fire an event.
 */
	public void reclusterEntities(Object source, InterfaceEntityList entities, boolean isEventFired) {
		InterfaceEntityList[] sortedEntities = entities.sortByCluster();
		Hashtable entityClusterMap = new Hashtable(sortedEntities.length * 2);		
		ClusterList oldClusters = entities.getAllClusters();
		InterfaceCluster oldCluster;
		int numEntityLists = sortedEntities.length;
		InterfaceEntityList currentList;
		for (int i = 0; i <= numEntityLists - 1; i++) {
			currentList = sortedEntities[i];
			oldCluster = currentList.elementAt(0).getParentCluster();
			oldCluster.removeEntities(currentList, false);
			entityClusterMap.put(oldCluster, currentList);
			}
		addEntities(entities, false);
		if (isEventFired) {
			eventManager.fireEntitiesReclustered(source, entities, oldClusters, this, entityClusterMap);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the cluster as added to the source
 * @param source an object to which the cluster has been added
 */
	public void setAdded(Object source) {
		eventManager.fireClusterAdded(source, this);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the cluster as removed from the source
 * @param source an object to which the cluster has been removed
 */
	public void setRemoved(Object source) {
		eventManager.fireClusterRemoved(source, this);
		InterfaceClusterManager.getInstance().unregisterCluster(this);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the cluster as selected.  If the cluster is already selected, another 
 * event does not fire.
 * @param source the object setting the cluster as selected
 */
	public void setSelected(Object source) {
		//if the cluster is already selected, do not fire another event
		if (!isSelected()) {
			isSelected = true;
			eventManager.fireClusterSelected(source, this);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the cluster as unselected.  If the cluster is already unselected, another 
 * event does not fire.
 * @param source the object setting the cluster as unselected
 */
	public void setUnselected(Object source) {
		//if the cluster is already unselected, do not fire another event
		if (isSelected()) {
			isSelected = false;
			eventManager.fireClusterUnselected(source, this);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the cluster as modified by the entity.
 * @param source the object setting the cluster as modified
 * @param enitty the entity that has either been removed or added
 * @param isAdded true if the entity was added, false if it was removed
 */
	public void setModified(Object source, InterfaceEntity entity, boolean isAdded) {
		eventManager.fireClusterModified(source, this, entity, isAdded);
		}
	}