/*
class org.datamanager.userinterface.cerulean.model.InterfaceEntityManager
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/
////////////////////////////////////////////////////////////////////////////////

package org.datamanager.userinterface.cerulean.model;

import java.io.Serializable;
import java.util.Vector;
import org.datamanager.userinterface.cerulean.controller.entity.InterfaceEntityManagerEntityListener;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.event.EntityListener;

/**
 * The EntityManager is in charge of tracking entities and their listeners.  It
 * also is in charge of the currently grouped entities.  Classes that wish to
 * register a listener with all entities should do it through this classes
 * addEntityListener method.
 */

public class InterfaceEntityManager implements Serializable {
	private static InterfaceEntityManager entityManager;
	private Vector listeners;
	private InterfaceEntityList groupedEntities;
	private Vector entities;
////////////////////////////////////////////////////////////////////////////////
	private InterfaceEntityManager() {
		InterfaceEntityManagerEntityListener entityListener = new InterfaceEntityManagerEntityListener(this);
		listeners = new Vector(10);
		entities = new Vector(20);
		addEntityListener(entityListener);
		groupedEntities = new InterfaceEntityList();
		}

////////////////////////////////////////////////////////////////////////////////
	public static InterfaceEntityManager getInstance() {
		if (entityManager == null) {
			entityManager = new InterfaceEntityManager();
			}
		return entityManager;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Registers an entity with the entity manager.  This adds all listeners 
 * currently registered with the manager.  Any future listeners to register will
 * be added to the entity.
 * @param entity the entity that is registering
 */
	public void registerEntity(InterfaceEntity entity) {
		entities.add(entity);
		for(int i = 0; i <= listeners.size()-1; i++) {
			EntityListener listener = (EntityListener)listeners.elementAt(i);
			entity.addEntityListener(listener);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Unregisters an entity from the entity manager.  This removes all listeners 
 * currently attached to the entity.
 * @param entity the entity that is unregistering
 */
	public void unregisterEntity(InterfaceEntity entity) {
		entities.remove(entity);
		for(int i = 0; i <= listeners.size()-1; i++) {
			EntityListener listener = (EntityListener)listeners.elementAt(i);
			entity.removeEntityListener(listener);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Adds an entity listener to all interface entities.
 */
	public void addEntityListener(EntityListener listener) {
		for(int i = 0; i <= entities.size()-1; i++) {
			InterfaceEntity entity = (InterfaceEntity)entities.elementAt(i);
			entity.addEntityListener(listener);
			}
		listeners.add(listener);
		}


////////////////////////////////////////////////////////////////////////////////
/**
 * Removes an entity listener from all interface entities.
 */
	public void removeEntityListener(EntityListener listener) {
		for(int i = 0; i <= entities.size()-1; i++) {
			InterfaceEntity entity = (InterfaceEntity)entities.elementAt(i);
			entity.removeEntityListener(listener);
			}
		listeners.remove(listener);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Ungroups any currently grouped entities and groups only the
 * specified entities
 * @param entities the entities to group
 */
	public void setGroupedEntities(InterfaceEntityList groupedEntities) {
		ungroupAllEntities();
		this.groupedEntities = groupedEntities;
		for (int i = 0; i <= groupedEntities.size() - 1; i++) {
			groupedEntities.elementAt(i).setSelected(this);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Ungroups any currently grouped entities and groups only the specified entity
 * @param entity the entity to group
 */
	public void setAsSelectedEntity(InterfaceEntity entity) {
		ungroupAllEntities();
		addToGroupedEntities(entity);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Unselect all currently selected entities.
 */
	public void ungroupAllEntities() {
		InterfaceEntityList entities = getGroupedEntities();
		for (int i = 0; i <= entities.size() - 1; i++) {
			System.out.println("ungrouping in EntityManager"+i);
			entities.elementAt(i).setUnselected(this);
			}
		groupedEntities = new InterfaceEntityList();
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Add entity to the list of currently grouped entities.
 * @param entity the entity to group
 */
	public void addToGroupedEntities(InterfaceEntity entity) {
		getGroupedEntities().addEntity(entity);
		entity.setSelected(this);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Remove entity from the list of grouped entities
 * @param entity the entity to remove
 */
	public void removeFromGroupedEntities(InterfaceEntity entity) {
		getGroupedEntities().removeEntity(entity);
		entity.setUnselected(this);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns a list of all currently grouped entities.
 * @return an InterfaceEntityList containing the grouped entities
 */
	public InterfaceEntityList getGroupedEntities() {
		return groupedEntities;
		}
	}