/*
class org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/

////////////////////////////////////////////////////////////////////////////////

package org.datamanager.userinterface.cerulean.model.data.storage;

import java.io.Serializable;
import java.util.Vector;
import org.datamanager.kernel.Entity;
import org.datamanager.userinterface.cerulean.model.InterfaceCluster;
import org.datamanager.userinterface.cerulean.model.InterfaceEntity;

////////////////////////////////////////////////////////////////////////////////
/**
 * A container class that is essentially a list of InterfaceEntities.  
 * InterfaceEntities may be added, and removed also.
 *
 * @see InterfaceEntity
 * @see InterfaceCluster
 *
 * @author Gordon Murphy, gpmurphy@indiana.edu
 * @author Bryan Dawson, bdawson@indiana.edu
 */
public class InterfaceEntityList implements Serializable {
	private Vector entities;
	private static final int DEFAULT_NUMBER_OF_CLUSTERS = 20;

////////////////////////////////////////////////////////////////////////////////
/**
 * Create a new InterfaceEntityList with the default length.
 */
	public InterfaceEntityList() {
		entities = new Vector(DEFAULT_NUMBER_OF_CLUSTERS);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Create a new InterfaceEntityList containing the specified 
 * entity array.  The entity array is converted into InterfaceEntities
 * and then stored in the array.
 */
	public InterfaceEntityList(Entity[] list) {
		this();
		Entity entity;
		InterfaceEntity cEntity;
		for (int i=0; i <= list.length - 1; i++) {
			entity = list[i];
			cEntity = new InterfaceEntity(entity);
			addEntity(cEntity);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Adds the specified entity to the list.
 * @param entity the entity to add
 */
	public void addEntity(InterfaceEntity entity) {
		entities.add(entity);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Append the specified list to the list.
 * @param entities the InterfaceEntityList to append to the end of this list
 */
	public void addEntities(InterfaceEntityList entities) {
		int numEntities = entities.size();
		for (int i = 0; i <= numEntities - 1; i++) {
			addEntity(entities.elementAt(i));
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes the specified entity from the list.
 * @param entity the entity to remove
 */
	public void removeEntity(InterfaceEntity entity) {
		entities.remove(entity);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes the specified list from this list.
 * @param entities the InterfaceEntityList to remove from this list
 */
	public void removeEntities(InterfaceEntityList entities) {
		int numEntities = entities.size();
		for (int i = 0; i <= numEntities - 1; i++) {
			removeEntity(entities.elementAt(i));
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieves the entity at the specified index.
 * @param index the index to retrieve the entity from
 * @return the entity at index
 */ 
	public InterfaceEntity elementAt(int index) {
		return (InterfaceEntity)entities.elementAt(index);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Inserts a entity at the specified index.
 * @param entity the entity to add
 * @param index the index to insert the entity at
 */
	public void insertElementAt(InterfaceEntity entity, int index) {
		entities.insertElementAt(entity, index);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes the entity at the specified index and returns it.
 * @param index the index to remove the entity at.
 * @return the entity removed
 */
	public InterfaceEntity removeElementAt(int index) {
		InterfaceEntity entity = elementAt(index);
		entities.removeElementAt(index);
		return entity;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * The number of entities in the list.
 * @retun the number of entities in the list
 */
	public int size() {
		return entities.size();
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Returns true if the list contains the specified entity.
 * @return true or false
 */
	public boolean contains(InterfaceEntity entity) {
		return entities.contains(entity);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Converts the contents to a string representation
 * @return a string representation of the contents
 */
	public String toString() {
		return entities.toString();
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sorts the entities in this list into the separate InterfaceEntityLists based
 * upon their parent clusters.
 * @return an array of InterfaceEntityLists.  Each list represents entities in
 *  a particular cluster.
 */
	public InterfaceEntityList[] sortByCluster() {
		ClusterList parentClusters = this.getAllClusters();
		int numEntities = this.size();
		InterfaceEntity currentEntity;
		InterfaceCluster currentCluster;
		
		int numClusters = parentClusters.size();
		InterfaceEntityList[] sortedEntities = new InterfaceEntityList[numClusters];
		for (int i = 0; i <= numClusters - 1; i++) {
			sortedEntities[i] = getAllEntitiesInCluster(parentClusters.elementAt(i));
			}
		return sortedEntities;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Creates a new InterfaceEntityList containing only entities in the specified
 * cluster.
 * @return an IntefaceEntityList containing only entities contained in the 
 *  specified cluster.
 */
	public InterfaceEntityList getAllEntitiesInCluster(InterfaceCluster cluster) {
		InterfaceEntityList list = new InterfaceEntityList();
		for (int i = 0; i <= size() - 1; i++) {
			if (elementAt(i).getParentCluster() == cluster) {
				list.addEntity(elementAt(i));
				}
			}
		return list;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieve all the different clusters who have entities contained in this
 * InterfaceEntityList.
 */
	public ClusterList getAllClusters() {
		ClusterList parentClusters = new ClusterList();
		int numEntities = this.size();
		InterfaceEntity currentEntity;
		InterfaceCluster currentParentCluster;
		for (int i = 0; i <= numEntities - 1; i++) {
			currentEntity = elementAt(i);
			currentParentCluster = currentEntity.getParentCluster();
			if (!parentClusters.contains(currentParentCluster)) {
				parentClusters.addCluster(currentParentCluster);
				}
			}
		return parentClusters;
		}
	}