package org.datamanager.userinterface.cerulean.model.undo;

import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CannotRedoException;
import org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoEdit;
import org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoManager;
import org.datamanager.userinterface.cerulean.model.InterfaceEntity;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.data.storage.ClusterList;
import org.datamanager.userinterface.cerulean.model.InterfaceCluster;

/**
 *This class holds the necessary information to undo/redo a reclustering of
 *InterfaceEntity's from multiple (or a single) InterfaceClusters.  It gets
 *most of its functionality (indirectly through InterfaceUndoEdit) from 
 *AbstractUndoableEdit.
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indiana.edu)
 *
 *@see org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoManager
 *@see org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoEdit
 *@see org.datamanager.userinterface.cerulean.model.InterfaceEntity
 *@see org.datamanager.userinterface.cerulean.model.InterfaceCluster
 *@see javax.swing.undo.AbstractUndoableEdit
 *@see org.datamanager.userinterface.cerulean.model.data.storage.ClusterList
 */
//  Methods:
//  ~~~~~~~~
//  public ReclusterUndoEdit(InterfaceEntity, ClusterList, InterfaceCluster)
//          This creates a ReclusterEntityUndoEdit for the corresponding 
//  	InterfaceEntity's.  The ClusterList represents the InterfaceClusters 
//  	that the InterfaceEntity's belonged to. The InterfaceCluster 
//  	represents the new cluster for the InterfaceEntity's.
//  public void undo() throws CannotUndoException
//          Adds the InterfaceEntity's  back to their old clusters.
//  public void Redo() throws CannotRedoException
//          Adds the InterfaceEntity's back to their new cluster        
//  public String toString()
//          returns the proper name from the history list to represent the edit
//          correctly.        
//  public void die()
//  	IMPORTANT:calls super.die()  otherwise it will not be removed from the 
//  	list correctly. Also sets up things to be garbage collected.
/////////////////////////////////////////////////////////////////////

public class ReclusterEntityUndoEdit extends InterfaceUndoEdit
   {
   //This is the InterfaceEntity that we want to undo/redo the addition of
   private InterfaceEntityList entities;

   //This is the InterfaceClusters that the InterfaceEntity's use to  belonged 
   //to, and the InterfaceCluster that they now belong to 
   private InterfaceCluster newCluster;
   private ClusterList oldClusters;
   
   //this will be the source for the EntityClusteringEvent's 
   private InterfaceUndoManager manager;

   ////////////////////////////////////////////////////////
   /**
    *Creates the undo edit for the reclustering of the given InterfaceEntity's
    *from the old InterfaceCluster's to the new InterfaceCluster.
    *@param entities The InterfaceEntity's that were reclustered.
    *@param oldClusters The old clusters that the InterfaceEntity's were removed from 
    *@param newCluster The ne cluster that the InterfaceEntity's were added to.
    */
   public ReclusterEntityUndoEdit(InterfaceEntityList entities, ClusterList oldClusters, InterfaceCluster newCluster)
      {
      this.entities = entities;
      this.newCluster = newCluster;
	this.oldClusters = oldClusters;
      manager = InterfaceUndoManager.getInstance();
      }

   ////////////////////////////////////////////////////////
   /** 
    *Redoes the reclustering.  Overriden from the
    *  superclass.  To insure that you do not get an exception
    * you should allows call the method canRedo() which returns
    *  a boolean saying if this edit can be redone at this time.
    */
   public void redo() throws CannotRedoException
      {
      //make sure all of the proper fields are set
      super.redo();
      //read the InterfaceEntity to the InterfaceCluster from which it came.
      newCluster.reclusterEntities(manager, entities, true);
      }

   ////////////////////////////////////////////////////////
   /**
    *Undoes the reclustering of the InterfaceEntity's (ie it adds them back 
    *to their orginal clusters).  Overriden from the
    * superclass.  To insure that you do not get an exception
    * you should allows call the method canUndo() which returns
    *  a boolean saying if this edit can be undone at this time.
    */
   public void undo() throws CannotUndoException
      {
      //make sure all of the proper fields are set
      super.undo();
      //remove the InterfaceEntity to the InterfaceCluster from which it came.
	InterfaceEntityList[] list = entities.sortByCluster();
	InterfaceCluster oldCluster;
	for (int i = 0; i <= list.length - 1; i++) {
		oldCluster = list[i].elementAt(0).getOldCluster();
		oldCluster.reclusterEntities(manager, list[i], true);
		}
      }

   ////////////////////////////////////////////////////////
   /**
    * Set the references  for garbage collection.  IMPORTANT: Calls 
    *super.die() so that it is removed from the HistoryList
    */
   public void die()
      {
      super.die();
      entities = null;
      newCluster = null;
	oldClusters = null;
	manager = null;
      }
   
   ///////////////////////////////////////////////////////////
   /** 
    *Returns the proper string to be displayed in the HistoryList
    *@return The string to be displayed. 
    */   
   public String toString()
      {
      if(this.canUndo())
      	 return new String("Undo Reclustering of Entities Containing: " +
			   entities.elementAt(0).getName());
      else
      	 return new String("Redo Reclustering of Entities Containging: " +
			   entities.elementAt(0).getName());
      }  

   }//end of class
