package org.datamanager.userinterface.cerulean.view.tree;

import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTree;
import java.awt.Component;
import java.awt.Color;

/**
 *This class responsible for renderering each cell of EntityTree for the user
 *to see.  It uses several icons to represent open (expanded) clusters,
 *closed (collapsed) clusters, entities (leaves), and empty clusters (which
 *are also leaves, but are their position is checked). It keeps everything
 *transparent for the majority of the time; only opaque when it is selected.
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indina.edu)
 *
 *@see org.datamanager.userinterface.cerulean.model.tree.EntityTreeModel
 *@see org.datamanager.userinterface.cerulean.view.tree.EntityTree
 */
//  Methods:
//  ~~~~~~~~
//  public InterfaceTreeCellRenderer()
//          creates a cell renderer with the default selection color of red.
//  public InterfaceTreeCellRenderer()
//          creates a cell renderer with the specified selection color.
//  public Component getTreeCellRendererComponent(JTree, Object,boolean,
//                                             boolean, boolean,int, boolean)
//          From TreeCellRenderer interface. This is the heart of this class,
//          the JTree calls this method for each cell of the tree.  It returns //          the compenent that is to be used for displaying the cell.
//  private void setAsEntityOrEmptyCluster(Object)
//          This method is passed the Object from the 
//          getTreeCellRendererComponent method.  If the Object is a 
//          DefaultMutableTreeNode, then it determines if it is an
//          entity(level of 2) or an empty cluster(level of 1).  If
//          it is not a DefaultMutableTreeNode, then it treats it as an entity.
///////////////////////////////////////////////////////////////////////

public class InterfaceTreeCellRenderer extends JLabel implements
						     TreeCellRenderer
   {

   private Color selectedColor;

   //the icons to represent entities and clusters
   private ImageIcon entityIcon;
   private ImageIcon openIcon;
   private ImageIcon closedIcon;
   private ImageIcon emptyIcon;
   private ImageIcon emptyRootIcon;
   private ImageIcon rootIcon;

   public final static Color DEFAULT_SELECTION_COLOR = new Color(206,206,255);

   ////////////////////////////////////////////////////////////////////
   //uses a default selection color of red
   //
   public InterfaceTreeCellRenderer()
      {
      this(DEFAULT_SELECTION_COLOR);
      }

   ////////////////////////////////////////////////////////////////////
   //uses the given color as the selection color
   //
   public InterfaceTreeCellRenderer(Color selectionColor)
      {
      this.selectedColor = selectionColor;

      //create the icons
      entityIcon = new ImageIcon("org/datamanager/userinterface/cerulean/view/tree/images/entity_c.gif");
      openIcon = new ImageIcon("org/datamanager/userinterface/cerulean/view/tree/images/opencluster_c.gif");
      closedIcon = new ImageIcon("org/datamanager/userinterface/cerulean/view/tree/images/closedcluster_c.gif");
      emptyIcon = new ImageIcon("org/datamanager/userinterface/cerulean/view/tree/images/emptycluster_c.gif");
      emptyRootIcon = new ImageIcon("org/datamanager/userinterface/cerulean/view/tree/images/root.gif");
      rootIcon = new ImageIcon("org/datamanager/userinterface/cerulean/view/tree/images/root_c.gif");									

      //make sure that we are transparent
      setOpaque(false);
      setForeground(Color.black);
      }

   ////////////////////////////////////////////////////////////////////////
   //From the TreeCellRenderer interface, this creates the componet that
   //will represent the cell.
   //
   public Component getTreeCellRendererComponent(JTree tree, Object value,
						boolean selected,
                                                boolean expanded,boolean leaf,
						int row, boolean hasFocus)
        {
        //set the icon for this cell
        if(leaf)
           {
           if(isRoot(value))
              setIcon(emptyRootIcon);
	   else	           	
              setAsEntityOrEmptyCluster(value); 
	   } 
	else if(isRoot(value))
	   setIcon(rootIcon);	                
        else if (expanded)
           setIcon(openIcon);
        else 
           setIcon(closedIcon);

        //now, check if it is selected. If so make it opaque.
        if(selected)
           {
           setOpaque(true);
           setBackground(selectedColor);
           }
        else
           setOpaque(false);
           
        
        setText(value.toString()+"  ");

        //Return this as the cell component to be displayed.
        return this;
	}

   //////////////////////////////////////////////////////////////////
   //This method is called if the cell to be displayed is a leaf.
   //
   private void setAsEntityOrEmptyCluster(Object value)
        {
        //If it is a DefaultMutableTreeNode, then can check if it is to
        //represent an entity (level 2) or just an empty cluster(level 1)
        if(value instanceof DefaultMutableTreeNode)
             {
             DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
             if(node.getLevel() == 1)
                setIcon(emptyIcon);
             else
                setIcon(entityIcon);
             }
        //however, if it is not then we must assume that it is an entity,
        //since we have no other way to check.
        else
           setIcon(entityIcon);
        }
        
   private boolean isRoot(Object value)
      {
      //if it is a DefaultMutableTreeNode, then we can determine its level
      if(value instanceof DefaultMutableTreeNode)
      	 {
      	 DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
      	 if(node.getLevel() == 0)
      	 	return true;
	 else
	 	return false;
	 }
       //otherwise we cannot assume anything, an must return false	 
      else
      	return false;	 	 	 	 	      	 	
      }	 
      	        
        
}	
						
