import org.datamanager.tools.Information;
import org.datamanager.tools.Debug;
import org.datamanager.collector.tools.WebConnection;
import org.datamanager.kernel.*;
import org.datamanager.passiveentityvalue.*;
import org.datamanager.exception.*;
import org.datamanager.event.*;
import org.datamanager.constraint.*;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;

	/**
	This Simpleton looks for an Entity that has a
	"content"/java.lang.String attribute but not a
	"url list"/java.util.Enumeration attribute. Once such an
	Entity is located it is processed.
	<p>
	During the processing the contents of the Entity is taken
	and all of the URLs located within it are parsed out. For
	each of these URLs a new Entity is created and put into the
	InfoPool. After all of the URLs are parsed out they are then
	added as an Enumeration in the "url list"/java.util.Enumeration
	attribute on the original Entity.
	<p>
	The new Entities that are put into the InfoPool are later
	located by the UrlDownloadSimpleton and downloaded.
	<p>
	Collaborators are the InfoPool and Entities.
	**/

public class WebParsingSimpleton
	extends Simpleton
	implements EventHandler
	{
	/*
	Author: Matthew Farrellee <mafarrel@cs.indiana.edu>
	
	API:
	public WebParsingSimpleton()
		This creates a new WebParsingSimpleton and sets
		up all the variables needed for it to process
		Entities.
		
	public void process(Entity entity)
		This method will process an Entity given to it.
	*/


	private static EntityConstraint unparsedMarker =
		new AndConstraint(
			new NameEqualsConstraint("ContentParsed"),
			new ValueEqualsConstraint(BooleanEntityValue.FALSE));

	private static EntityConstraint unparsedEntity =
			new AttributeExistsConstraint(unparsedMarker);


	////////////////////////////////////////////////////////////////////
	
	/**
	This is the Constructor. It creates a new WebParsingSimpleton and
	sets up all the variables needed for processing of Entities. The
	variable setup is done in: the registerAttributes() method, which
	registers all the AttributeTypes needed by this Simpleton; the
	getAttributeTypes() method, which initializes all the AttributeType
	variables located within this Simpleton; and the setConstraint()
	method, which sets the Constraint, or predicate, of this Simpleton.
	**/
	
	public WebParsingSimpleton()
		{
		}

	////////////////////////////////////////////////////////////////////

	public Information getInformation()
		{
		String[] authors = {""};
		return new Information("WebParsingSimpleton",
										"short", "long", "version", authors);
		}

	////////////////////////////////////////////////////////////////////

	/**
	This method processes an Entity. It takes the Entity and using the
	parseUrlsFrom() method extracts a Vector of all the Urls in the
	contents of the Entity. It then takes that Vector of Urls, turns
	it into an Enumeration, and adds it as the value of the Entity's
	urlListAttribute.
	
	@param entity An Entity to process.
	**/

	public void process()
		{
		try
			{
			EventConstraint constraint =
				new AddedEntitySatisfiesConstraint(unparsedEntity);
			Pool.getDefaultPool().subscribe(this, constraint);
			}
		catch (DataManagerException exception)
			{
			// Cannot subscribe().
			}
		}

	public void handle(DataManagerEvent event)
		{
		try
			{
			WebConnection connection = WebConnection.getConnection();
		
			AddEntityEvent addEvent =
				(AddEntityEvent) event;

			Entity entity = addEvent.getAddedEntity();

			// Use the parseUrlFrom() method to generate a list of Urls
			// from the contents of the entity.
			Vector urlList = connection.getURLs(entity.getValue().toString());

			Enumeration urls = urlList.elements();
			URL url;
			while (urls.hasMoreElements())
				{
				try
					{
					url = (URL) urls.nextElement();
					Entity newEntity = Entity.create(url.toExternalForm());
					Entity urlAttribute = Entity.create("content-location");
					
					newEntity.setValue(new NullEntityValue());
					// This really should be a URLEntityValue.
					urlAttribute.setValue(new StringEntityValue(url.toExternalForm()));
					
					newEntity.attach(urlAttribute);	

					entity.attach(newEntity);
					Pool.getDefaultPool().add(newEntity);
					}
				catch (DataManagerException exception)
					{
					// No error should appear in Hydrogen.
					}
				}

			Entity[] attributes = entity.getAttributes(unparsedMarker);
			for (int i = 0; i < attributes.length; i++)
				{
				try
					{
					attributes[i].setValue(BooleanEntityValue.TRUE);
					}
				catch (DataManagerException exception)
					{
					// setValue() failed.
					}
				}
			}
		catch (DataManagerException exception)
			{
			// Nuttin'
			}
		}
	}
