/**
 * WebSearchSimpleton.java
 */

import org.datamanager.collector.tools.WebConnection;
import org.datamanager.event.*;
import org.datamanager.constraint.*;
import org.datamanager.exception.*;
import org.datamanager.passiveentityvalue.*;
import org.datamanager.kernel.Simpleton;
import org.datamanager.tools.Information;
import org.datamanager.kernel.Entity;
import org.datamanager.kernel.*;
import org.datamanager.tools.Debug;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;


/**
 * This Simpleton hangs around until it gets a WebSearchEvent.
 * Once it has received the event it tries to form a web query,
 * a query to a search engine such as webcrawler, and the
 * resulting page is downloaded and added to the pool.
 * 
 * @see org.datamanager.tools.Debug;
 * @see java.io.IOException;
 * @see java.net.MalformedURLException;
 * @see java.net.URL;
 * @see java.net.URLEncoder;
 *
 * @author Matthew Farrellee
 */


public class WebSearchSimpleton
	extends Simpleton
	implements EventHandler
	{
	/**
	 * This is the default constructor. It calls the constructor that
	 * takes a String with a default name.
	 */
	public WebSearchSimpleton()
		{
		}

	////////////////////////////////////////////////////////////////////

	public void process()
		{
		try
			{
			EventConstraint constraint =
				new EventTypeIsConstraint(WebSearchEvent.class);
			Pool.getDefaultPool().subscribe(this, constraint);
			}
		catch (DataManagerException exception)
			{
			}
		}

	public Information getInformation()
		{
		String[] authors = {""};
		return new Information("WebSearchSimpleton",
										"short", "long", "version", authors);
		}


	public void handle(DataManagerEvent event)
		{
		WebSearchEvent search = (WebSearchEvent) event;
		download(search.getEngine(), search.getKeywords());
		}

	////////////////////////////////////////////////////////////////////

	/**
	This method takes an Entity and modifies it. First by downloading
	the contents of the URL which is attached to it, and then by
	attaching that contents to the Entity. This method is destructive
	because there is no need to create a new Entity if the one passed
	to it can be modified just as easily.
	This method is called from the super class's process() method.
	
	@param entity An entity to update/modify.
	**/

	protected void download(String engine, String keywords)
		{
		// getContentsFromURL can throw an IOException.
		try
			{
			WebConnection connection = WebConnection.getConnection();
			
			// Create the query URL.
			// Use URLEncoder to encode the url.
			URL url = new URL(engine + URLEncoder.encode(keywords));

			// Turn the URL into a String. Not the actual URL itself,
			// but rather the contents that it points to. Usually a
			// web page.
			String contents = connection.getContents(url);
			
			System.err.println("Got contents from: " + url);

			Entity searchResults = Entity.create(keywords);
			Entity keywordsAttribute = Entity.create("SearchKeywords");
			Entity engineAttribute = Entity.create("SearchEngine");
			Entity mimeType = Entity.create("mime-type");
			Entity contentType = Entity.create("content-source");

			Entity searchMark = Entity.create("ContentParsed");

			searchMark.setValue(BooleanEntityValue.FALSE);
			
			keywordsAttribute.setValue(new StringEntityValue(keywords));
			engineAttribute.setValue(new StringEntityValue(engine));
			contentType.setValue(new StringEntityValue("www"));
			mimeType.setValue(new StringEntityValue("text/html"));
			searchResults.setValue(new StringEntityValue(contents));

			searchResults.attach(keywordsAttribute);
			searchResults.attach(mimeType);
			searchResults.attach(contentType);

			searchResults.attach(searchMark);

			Pool.getDefaultPool().add(searchResults);

			System.err.println("Added search result entity to pool.");

			DocumentDownloadedEvent downloadEvent = new DocumentDownloadedEvent(Pool.getDefaultPool(), searchResults);
			Pool.getDefaultPool().add(downloadEvent);

			}
		catch (DataManagerException exception)
			{
			}
		catch (MalformedURLException badUrl)
			{
			// Forget about this Entity and wait for another.
			Debug.assert("Problem making the keyword into a URL: " + badUrl);
			}
		// If there is trouble getting the contents from the URL.
		catch (IOException ioError)
			{
			// Signal that there was a problem with an assertion.
			Debug.assert("Got an IOException: " + ioError.getMessage());
			}
		}
	} // End of SearchSimpleton class.
